"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TypescriptCompiler = void 0;

function _path() {
  const data = _interopRequireDefault(require("path"));

  _path = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function _typescript() {
  const data = _interopRequireDefault(require("typescript"));

  _typescript = function () {
    return data;
  };

  return data;
}

class TypescriptCompiler {
  constructor(tsConfig) {
    this.tsConfig = tsConfig;
  }

  compileFile(fileContent, options) {
    const supportedExtensions = ['.ts', '.tsx'];

    const fileExtension = _path().default.extname(options.filePath);

    if (!supportedExtensions.includes(fileExtension)) {
      return null; // file is not supported
    }

    const compilerOptionsFromTsconfig = _typescript().default.convertCompilerOptionsFromJson(this.tsConfig.compilerOptions, '.');

    if (compilerOptionsFromTsconfig.errors.length) {
      throw new Error(`failed parsing the tsconfig.json.\n${compilerOptionsFromTsconfig.errors.join('\n')}`);
    }

    const compilerOptions = compilerOptionsFromTsconfig.options;
    compilerOptions.sourceRoot = options.componentDir;

    const result = _typescript().default.transpileModule(fileContent, {
      compilerOptions,
      fileName: options.filePath,
      reportDiagnostics: true
    });

    if (result.diagnostics && result.diagnostics.length) {
      const formatHost = {
        getCanonicalFileName: p => p,
        getCurrentDirectory: _typescript().default.sys.getCurrentDirectory,
        getNewLine: () => _typescript().default.sys.newLine
      };

      const error = _typescript().default.formatDiagnosticsWithColorAndContext(result.diagnostics, formatHost);

      throw new Error(error);
    }

    const replaceExtToJs = filePath => filePath.replace(new RegExp(`${fileExtension}$`), '.js'); // makes sure it's the last occurrence


    const outputPath = replaceExtToJs(options.filePath);
    const outputFiles = [{
      outputText: result.outputText,
      outputPath
    }];

    if (result.sourceMapText) {
      outputFiles.push({
        outputText: result.sourceMapText,
        outputPath: `${outputPath}.map`
      });
    }

    return outputFiles;
  }

  compileOnCapsules(capsuleDirs) {
    capsuleDirs.forEach(capsuleDir => _fsExtra().default.writeFileSync(_path().default.join(capsuleDir, 'tsconfig.json'), JSON.stringify(this.tsConfig, undefined, 2)));

    const compilerOptionsFromTsconfig = _typescript().default.convertCompilerOptionsFromJson(this.tsConfig.compilerOptions, '.');

    if (compilerOptionsFromTsconfig.errors.length) {
      throw new Error(`failed parsing the tsconfig.json.\n${compilerOptionsFromTsconfig.errors.join('\n')}`);
    }

    const diagnostics = [];

    const diagAccumulator = diag => diagnostics.push(diag);

    const host = _typescript().default.createSolutionBuilderHost(undefined, undefined, diagAccumulator);

    const solutionBuilder = _typescript().default.createSolutionBuilder(host, capsuleDirs, {
      dry: false,
      verbose: false
    });

    solutionBuilder.clean();
    const result = solutionBuilder.build();
    let errorStr = '';

    if (diagnostics.length) {
      const formatHost = {
        getCanonicalFileName: p => p,
        // @todo: replace this with the capsule dir for better error message
        getCurrentDirectory: _typescript().default.sys.getCurrentDirectory,
        getNewLine: () => _typescript().default.sys.newLine
      };
      errorStr = _typescript().default.formatDiagnosticsWithColorAndContext(diagnostics, formatHost);
    }

    return {
      resultCode: result,
      error: errorStr ? new Error(errorStr) : null
    };
  }

}

exports.TypescriptCompiler = TypescriptCompiler;